using System;
using System.IO;
using System.Text;
using System.Linq;

namespace MxAPITest
{
    class Program
    {
        static void RunMxFrameTest(string strProject)
        {
            Console.Out.WriteLine("Creating MxFrameAPI");

            if (Mx.Frame.CreateConsole())
            //if (Mx.Frame.Create((IntPtr)null))
            {                
                Console.Out.WriteLine(String.Format("Loading project file {0}", strProject));
                if (Mx.Frame.Load(strProject))
                {
                    string strStructure = Mx.Frame.ExportStructure();
                    Console.Out.WriteLine(String.Format("Structure exported into {0}", strStructure));

                    //string strProjectNew = strProject + ".save.mxml";
                    //if (Mx.Frame.Save(strProjectNew))
                    //    Console.Out.WriteLine(String.Format("Project saved into {0}", strProjectNew));
                    //else
                    //    Console.Out.WriteLine(String.Format("Failed to save project into {0}", strProjectNew));

                    //Console.ReadLine();

                    Console.Out.WriteLine("Calculating project");
                    if (Mx.Frame.Calculate())
                    {
                        string strLog = Mx.Frame.ExportLog("CALC");
                        Console.Out.WriteLine(String.Format("Calculation log exported into {0}", strLog));
                        string strResults = Mx.Frame.ExportData(true, Mx.Frame.ETables.ET_FULL);
                        Console.Out.WriteLine(String.Format("Results exported into {0}", strResults));

                        //Console.Out.WriteLine("Calculating steel");
                        //string strSteel = Mx.Frame.CalculateSteel();
                        //Console.Out.WriteLine(String.Format("Steel data exported into {0}", strSteel));

                        //Console.Out.WriteLine("Calculating timber");
                        //string strTimber = Mx.Frame.CalculateTimber();
                        //Console.Out.WriteLine(String.Format("Timber data exported into {0}", strTimber));

                        //Console.Out.WriteLine("Calculating concrete");
                        //string strConcrete = Mx.Frame.CalculateConcrete();
                        //Console.Out.WriteLine(String.Format("Concrete data exported into {0}", strConcrete));
                    }
                    else
                        Console.Out.WriteLine("Failed to calculate");

                    //string strReport = Mx.Frame.MakeReport("Structure-Member,Structure-Section,Structure-Release,Structure-Support");
                    //Console.Out.WriteLine(String.Format("Report generated into {0}", strReport));
                }
                else
                    Console.Out.WriteLine("Failed to load project");

                Console.Out.WriteLine("Destroying MxFrameAPI");
                if (!Mx.Frame.Destroy())
                    Console.Out.WriteLine("Failed to destroy MxFrameAPI");
            }
            else
                Console.Out.WriteLine("Failed to create MxFrameAPI");
        }

        static void RunMxBeamTest(string strProject)
        {
            Console.Out.WriteLine("Creating MxBeamAPI");

            if (Mx.Beam.Create())
            {

                Console.Out.WriteLine(String.Format("Loading project file {0}", strProject));
                if (Mx.Beam.LoadProject(strProject))
                {
                    Console.Out.WriteLine("Calculating project");
                    if (Mx.Beam.ResultsCalculate())
                    {
                        string strResults = Mx.Beam.ExportData();
                        Console.Out.WriteLine(String.Format("Results exported into {0}", strResults));
                        
                        //string strReport = Mx.Beam.MakeReport();

                        //if (strReport.Length > 0)
                        //    Console.Out.WriteLine(String.Format("Generated report file {0}", strReport));
                        //else
                        //    Console.Out.WriteLine("Failed to generate report");
                    }
                    else
                        Console.Out.WriteLine("Failed to calculate");
                }
                else
                    Console.Out.WriteLine("Failed to load project");

                Console.Out.WriteLine("Destroying MxBeamAPI");
                if (!Mx.Beam.Destroy())
                    Console.Out.WriteLine("Failed to destroy MxBeamAPI");
            }
            else
                Console.Out.WriteLine("Failed to create MxBeamAPI");
        }

        static void RunMxToolsTest(string strPath, string strDocID)
        {
            Console.Out.WriteLine("Creating MxToolsAPI");

            if (Mx.Tools.Create())
            {
                Console.Out.WriteLine(strPath);

                int idDoc;

                if (!Int32.TryParse(strDocID, out idDoc))
                {
                    idDoc = 2000;
                }

                if (System.IO.File.Exists(strPath))
                {
                    string jsonData = System.IO.File.ReadAllText(strPath);

                    Console.Out.WriteLine(String.Format("Document ID: {0}", idDoc));
                    Console.Out.WriteLine(String.Format("Document Data: {0}", jsonData));

                    string jsonReturn = Mx.Tools.CallDocument(idDoc, jsonData);

                    Console.Out.WriteLine(jsonReturn);
                }
                else
                {
                    Console.Out.WriteLine(String.Format("Cannot find file: {0}", strPath));
                }
            }
            else
                Console.Out.WriteLine("Failed to create MxToolsAPI");
        }

        static void RunMxToolsPoolTest(string strPath, string strDocID, string code, string lng)
        {
            Console.Out.WriteLine("Creating MxToolsAPI Pool");

            if (Mx.ToolsPool.Create())
            {
                Console.Out.WriteLine(strPath);

                int idDoc;

                if (!Int32.TryParse(strDocID, out idDoc))
                {
                    idDoc = 2000;
                }

                if (System.IO.File.Exists(strPath))
                {
                    string jsonData = System.IO.File.ReadAllText(strPath);

                    Console.Out.WriteLine(String.Format("Document ID: {0}", idDoc));
                    Console.Out.WriteLine(String.Format("Document Data: {0}", jsonData));

                    string jsonReturn = Mx.ToolsPool.CallDocument(code, lng, idDoc, jsonData);

                    Console.Out.WriteLine(jsonReturn);
                }
                else
                {
                    Console.Out.WriteLine(String.Format("Cannot find file: {0}", strPath));
                }

                if (!Mx.ToolsPool.Destroy())
                    Console.Out.WriteLine("Failed to destroy MxToolsAPI Pool");
            }
            else
                Console.Out.WriteLine("Failed to create MxToolsAPI Pool");
        }

        static void RunMxFramePoolTest(string strPath, string code, string lng)
        {
            Console.Out.WriteLine("Creating MxFrameAPI Pool");

            if (Mx.FramePool.Create())
            {
                MxTSurrogate560Lib.MxFrame560 api = Mx.FramePool.GetAPI(code, lng);

                if (null != api)
                {
                    Console.Out.WriteLine(String.Format("Loading project file {0}", strPath));
                    if (0 != api.Load(strPath))
                    {
                        Console.Out.WriteLine("Calculating project");
                        if (0 != api.Calculate(0))
                        {
                            string strResults = api.ExportData(1, (int)Mx.Frame.ETables.ET_FULL);
                            Console.Out.WriteLine(String.Format("Results exported into {0}", strResults));
                        }
                        else
                            Console.Out.WriteLine("Failed to calculate");
                    }
                    else
                        Console.Out.WriteLine("Failed to load project");

                    Console.Out.WriteLine("Destroying MxFrameAPI Pool");
                    if (!Mx.FramePool.Destroy())
                        Console.Out.WriteLine("Failed to destroy MxFrameAPI Pool");
                }
                else
                    Console.Out.WriteLine("API not created");
            }
            else
                Console.Out.WriteLine("Failed to create MxFrameAPI Pool");
        }

        static void Main(string[] args)
        {
            bool bOK = false;
            long nCount = null != args ? args.Count() : 0;

            //Console.Out.WriteLine("Press any key to continue");
            //Console.ReadLine();

            if (nCount > 1)
            {
                switch (args[0].Trim().ToUpper())
                {
                    case "/B":
                        RunMxBeamTest(args[1].Trim());
                        bOK = true;
                        break;

                    case "/F":
                        RunMxFrameTest(args[1].Trim());
                        bOK = true;
                        break;

                    case "/T":
                        RunMxToolsTest(args[1].Trim(), nCount > 2 ? args[2].Trim() : "");
                        bOK = true;
                        break;

                    case "/TP":
                        RunMxToolsPoolTest(args[1].Trim(), nCount > 2 ? args[2].Trim() : "", nCount > 3 ? args[3].Trim() : "en-nen", nCount > 4 ? args[4].Trim() : "us");
                        bOK = true;
                        break;

                    case "/FP":
                        RunMxFramePoolTest(args[1].Trim(), nCount > 2 ? args[2].Trim() : "en-nen", nCount > 3 ? args[3].Trim() : "us");
                        bOK = true;
                        break;
                }
            }

            /*
            /F "d:\_incoming\_apis\MxFrameAPI.mxml"
            /TP "d:\_incoming\_apis\MxToolsAPICallDoc.json" 2001 en-nbn nl
            /FP "d:\_incoming\_apis\MxFrameAPI.mxml" en-nbn nl
            */

            if (!bOK)
            {
                Console.WriteLine("Error, wrong arguments!");
                Console.WriteLine("Usage:");
                Console.WriteLine("\tMxAPITest /B|F|T \"path to file\"");
                Console.WriteLine("\r\nSamples:");
                Console.WriteLine("\tMxAPITest /F \"path_to_mxframe_project.mxml\"");
                Console.WriteLine("\tMxAPITest /B \"path_to_mxbeam_project.mxml\"\r\n");
                Console.WriteLine("\tMxAPITest /T \"path_to_mxtools_input_data.json\" DOCID\r\n");
                Console.WriteLine("\tMxAPITest /TP \"path_to_mxtools_input_data.json\" DOCID CODE LANGUAGE\r\n");
                Console.WriteLine("\tMxAPITest /FP \"path_to_mxframe_project.json\" CODE LANGUAGE\r\n");
            }
                       
            Console.Out.WriteLine("Press any key to close the app");
            Console.ReadLine();
        }
    }
}