﻿using System;
using System.Collections.Generic;

namespace Mx
{
    public sealed class Frame
    {
        static private MxTSurrogate560Lib.MxFrame560 m_Frame = null;

        public enum ETables
        {
            ET_NODALFORCES = 1,
            ET_MEMBERFORCES = 1 << 2,
            ET_DISPLDEFL = 1 << 3,
            ET_SUPPORTREACTIONS = 1 << 4,
            ET_FORCECUTS = 1 << 5,
            ET_DISPLCUTS = 1 << 6,
            ET_ENVELOPE = 1 << 7,
            ET_POLYNOMS = 1 << 8,
            ET_STRESSES = 1 << 9,
            ET_DEFAULT = ET_NODALFORCES | ET_MEMBERFORCES | ET_DISPLDEFL | ET_SUPPORTREACTIONS,
            ET_FULL = 0xFFFF,
        };

        public enum EStructureCompare
        {
            ECF_EQUAL = 0,
            ECF_DIFF_STRUCTURE = 1,
            ECF_DIFF_LOADS = 1 << 1,
            ECF_ERROR = 1 << 2,

            ECF_DIFF_FULL = ECF_DIFF_STRUCTURE | ECF_DIFF_LOADS,
        };

        public static bool Create(IntPtr HWnd)
        {
            if (null == m_Frame)
                m_Frame = new MxTSurrogate560Lib.MxFrame560();

            return 0 != m_Frame.Create("", (int)HWnd);
        }

        public static bool CreateConsole()
        {
            if (null == m_Frame)
                m_Frame = new MxTSurrogate560Lib.MxFrame560();

            return 0 != m_Frame.CreateConsole("");
        }

        public static bool Destroy()
        {
            return 0 != m_Frame.Destroy();
        }

        public static bool Load(string path)
        {
            return 0 != m_Frame.Load(path);
        }

        public static bool LoadXML(string xml)
        {
            return 0 != m_Frame.LoadXML(xml);
        }

        public static bool Save(string path)
        {
            return 0 != m_Frame.Save(path);
        }

        public static string SaveXML()
        {
            return m_Frame.SaveXML();
        }

        public static bool Calculate(bool bAdvanced = false)
        {
            return 0 != m_Frame.Calculate(bAdvanced ? 1 : 0);
        }

        public static string ExportStructure(bool bToFile = true)
        {
            return m_Frame.ExportStructure(bToFile ? 1 : 0);
        }

        public static bool ImportStructure(string structure)
        {
            return 0 != m_Frame.ImportStructure(structure);
        }

        public static int CompareStructure(string structure)
        {
            return m_Frame.CompareStructure(structure);
        }

        public static string ExportData(bool bToFile = true, ETables eTables = ETables.ET_DEFAULT)
        {
            return m_Frame.ExportData(bToFile ? 1 : 0, (int)eTables);
        }

        public static string ExportLog(string id, bool bToFile = true)
        {
            return m_Frame.ExportLog(id, bToFile ? 1 : 0);
        }

        public static bool SetLoadsGeneratorSettings(string xmlSettings)
        {
            return 0 != m_Frame.SetLoadsGeneratorSettings(xmlSettings);
        }
        public static bool RunLoadsGenerator()
        {
            return 0 != m_Frame.RunLoadsGenerator();
        }

        static public string ShowSectionDlg(string strSection)
        {
            return m_Frame.ShowSectionDlg(strSection);
        }

        enum SteelChecks // same as in Redist\Steel\SteCalc\SteEnums.h
        {
            Steel_None_Check = 0,
            Steel_Cross_Check = (1 << 0),
            Steel_LatBuckling_Check = (1 << 1),
            Steel_Buckling_Check = (1 << 2),
            Steel_Torsion_Check = (1 << 3),
            Steel_Deflections_Check = (1 << 4),
            Steel_Fire_Check = (1 << 5),

            Steel_All_Buckling = (Steel_LatBuckling_Check | Steel_Buckling_Check),
            Steel_All_Buckling_Cross = (Steel_All_Buckling | Steel_Cross_Check),
            Steel_All_Buckling_Torsion = (Steel_All_Buckling | Steel_Torsion_Check),
            Steel_All_Buckling_Cross_Torsion = (Steel_All_Buckling_Cross | Steel_Torsion_Check),
            Steel_All_Check = (Steel_All_Buckling_Cross | Steel_Deflections_Check | Steel_Fire_Check),
        };

        public static string CalculateSteel(int nChecks = (int)SteelChecks.Steel_All_Check, bool bToFile = true)
        {
            return m_Frame.CalculateSteel(nChecks, bToFile ? 1 : 0);
        }

        enum TimberChecks // same as in Redist\FrameWork\FrameWork\Timber\TimberData.h
        {
            E_NO_CHECK = 1, ///< Check not defined
            E_CROSS_CHECK = 2,  ///< Cross check
            E_LATERAL_BUCKLING_CHECK = 4,   ///< Lateral buckling check
            E_BUCKLING_CHECK = 8,   ///< Buckling check
            E_DEFLECTIONS_CHECK = 16,	///< Deflections check

            E_FULL_CHECK = E_CROSS_CHECK | E_CROSS_CHECK | E_LATERAL_BUCKLING_CHECK | E_DEFLECTIONS_CHECK,
        }

        public static bool ShowSteelGrids()
        {
            return 0 != m_Frame.ShowSteelGrids();
        }

        public static string CalculateTimber(int nChecks = (int)TimberChecks.E_FULL_CHECK, bool bToFile = true)
        {
            return m_Frame.CalculateTimber(nChecks, bToFile ? 1 : 0);
        }

        public static bool ShowTimberGrids()
        {
            return 0 != m_Frame.ShowTimberGrids();
        }

        enum ConcreteFlags // same as in Redist\FrameWork\FrameWork\Concrete\ConExportUtils.h
        {
            E_CROSSCHECK = 1,
            E_BARPLACEMENT = 2,
            E_DEFLECTIONS = 4,
            E_ALLCONCRETE = 0xFFF,
        }

        public static string CalculateConcrete(int nFlags = (int)ConcreteFlags.E_ALLCONCRETE, bool bToFile = true)
        {
            return m_Frame.CalculateConcrete(nFlags, bToFile ? 1 : 0);
        }

        public static string MakeReport(string items)
        {
            return m_Frame.MakeReport(items);
        }

        public static bool DongleIsModule(short nModule)
        {
            return 0 != m_Frame.DongleIsModule(nModule);
        }
    };

    public sealed class FramePool
    {
        static private Dictionary<string, MxTSurrogate560Lib.MxFrame560> m_Pool;

        public static string GetKey(string code, string lng)
        {
            return String.Format("{0}|{1}", code, lng).ToLower();
        }

        public static string GetSettings(string code, string lng)
        {
            return "{ \"settings\": { \"code\": { \"general\": \"" + code + "\" }, \"language\": \"" + lng + "\" } }";
        }

        public static MxTSurrogate560Lib.MxFrame560 GetAPI(string code, string lng)
        {
            return m_Pool[FramePool.GetKey(code, lng)];
        }

        public static bool Create()
        {
            if (null == m_Pool)
                m_Pool = new Dictionary<string, MxTSurrogate560Lib.MxFrame560>();
            else
                m_Pool.Clear();

            string[] codes = { "en-nen", "en-nbn" };
            string[] lngs = { "us", "nl" };

            foreach (string code in codes)
            {
                foreach (string lng in lngs)
                {
                    string key = FramePool.GetKey(code, lng);
                    m_Pool[key] = new MxTSurrogate560Lib.MxFrame560();

                    Console.Out.WriteLine(String.Format("Creating api for key: {0}", key));
                    if (0 == m_Pool[key].CreateConsole(FramePool.GetSettings(code, lng)))
                    {
                        return false;
                    }
                }
            }

            return true;
        }

        public static bool Destroy()
        {
            foreach (KeyValuePair<string, MxTSurrogate560Lib.MxFrame560> i in m_Pool)
            {
                i.Value.Destroy();
            }

            m_Pool.Clear();
            m_Pool = null;
            return true;
        }
    }
}