﻿using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;

namespace Mx
{
    public sealed class Tools
    {
        static private MxTSurrogate560Lib.MxTools560 m_Tools = null;

        public static bool Create()
        {
            if (null == m_Tools)
                m_Tools = new MxTSurrogate560Lib.MxTools560();

            return 0 != m_Tools.CreateToolbox("");
        }

        public static string CallDocument(int nDocID, string strData)
        {
            return m_Tools.CallDocument(nDocID, strData);
        }
    };

    public sealed class ToolsPool
    {
        static private Dictionary<string, MxTSurrogate560Lib.MxTools560> m_Pool;

        public static string GetKey(string code, string lng)
        {
            return String.Format("{0}|{1}", code, lng).ToLower();
        }

        public static string GetSettings(string code, string lng)
        {
            return "{ \"settings\": { \"code\": { \"general\": \"" + code + "\" }, \"language\": \"" + lng + "\" } }";
        }

        public static MxTSurrogate560Lib.MxTools560 GetAPI(string code, string lng)
        {
            return m_Pool[ToolsPool.GetKey(code, lng)];
        }

        public static bool Create()
        {
            if (null == m_Pool)
                m_Pool = new Dictionary<string, MxTSurrogate560Lib.MxTools560>();
            else
                m_Pool.Clear();

            string[] codes = { "en-nen", "en-nbn" };
            string[] lngs = { "us", "nl" };

            foreach (string code in codes)
            {
                foreach (string lng in lngs)
                {
                    string key = ToolsPool.GetKey(code, lng);
                    m_Pool[key] = new MxTSurrogate560Lib.MxTools560();

                    Console.Out.WriteLine(String.Format("Creating api for key: {0}", key));
                    if (0 == m_Pool[key].CreateToolbox(ToolsPool.GetSettings(code, lng)))
                    {
                        return false;
                    }
                }
            }

            return true;
        }

        public static bool Destroy()
        {
            //foreach (KeyValuePair<string, MxTSurrogate560Lib.MxTools560> i in m_Pool)
            //{
            //    i.Value.Destroy();
            //}

            m_Pool.Clear();
            m_Pool = null;
            return true;
        }

        public static string CallDocument(string code, string lng, int nDocID, string strData)
        {
            MxTSurrogate560Lib.MxTools560 tools = ToolsPool.GetAPI(code, lng);
            return null != tools ? tools.CallDocument(nDocID, strData) : "";
        }
    }
}